SystemOrganization addCategory: #Reflectogram!
SystemOrganization addCategory: 'Reflectogram-Benchmarks'!
SystemOrganization addCategory: 'Reflectogram-Case-Study'!
SystemOrganization addCategory: 'Reflectogram-Context-Process-Add-Ons'!
SystemOrganization addCategory: 'Reflectogram-Events'!
SystemOrganization addCategory: 'Reflectogram-Graph-Points'!
SystemOrganization addCategory: 'Reflectogram-Interpreter-MObjects'!
SystemOrganization addCategory: 'Reflectogram-Modes'!
SystemOrganization addCategory: 'Reflectogram-Targets'!
SystemOrganization addCategory: 'Reflectogram-Tests'!
SystemOrganization addCategory: 'Reflectogram-Tests-Vanilla'!

TestCase subclass: #ContextEventTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Context-Process-Add-Ons'!

!ContextEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:06'!
testAContext

	|targetProcess targetContext conditionCounter actionCounter shouldNotImplement|

	targetProcess := [DummyEventTestingClass new aMethodWithSuspendedContext] fork.
	(Delay forSeconds: 2) wait.
	targetContext := targetProcess suspendedContext.

	conditionCounter := 0.
	actionCounter := 0.  
	shouldNotImplement := false. 

	[ReflectoGram 
		on: targetContext
		as: Entity -> Flat
		for: MSGSend 
		limitedTo: [ :method | true ]
		when: [ :rPoint | conditionCounter := conditionCounter + 1 . true ]
		do: [ :rPoint |  actionCounter := actionCounter + 1. rPoint returnValue: rPoint defaultAction value]] 
						on: ShouldNotImplement do: [ 
							Transcript show: 'DNI: '; show: targetContext class; show: '->'; show: self class; cr. shouldNotImplement := true ].  
						
	targetProcess resume.
	
	(Delay forSeconds: 2) wait.
	
	self assert: (conditionCounter = 1).
	self assert: (actionCounter = 1)! !

TestCase subclass: #EventTest
	instanceVariableNames: 'link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!EventTest class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:35'!
isAbstract

	^self name = #EventTest! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:02'!
event

	^ Event new
! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 12:34'!
expectedResultFor: aMethod

	|testObj result|
	
	testObj := self testObjectFor: aMethod.
	[ result := testObj perform: aMethod selector withEnoughArguments: { }] on: Exception do: [:ex | result := ex  ].
	
	^ result! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 14:40'!
methods

	^ { }! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/19/2015 14:27'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: aBlock;
		selector: #value;
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 9/4/2015 16:20'!
tearDown

	"additonal clean up"	
	
	link ifNotNilDo: [:l | l uninstall].
	self methods do: [:method | method recompile].	 "clean up and revert method to match each source-code"
	super tearDown.! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/6/2015 10:59'!
testEvent

	| expectedResult reflectiveResult metaJumpOccured |
	self methods do: [:method |
		expectedResult := self expectedResultFor: method.
		link := self newLinkWith: [metaJumpOccured := true].  
		self event 
			annotateMethod: method
			withLink: link.
			reflectiveResult := self expectedResultFor: method.
			link uninstall.
			method recompile.	"clean up and revert method to match each source-code"
			self assert: metaJumpOccured = true. "some events like sends may be triggered more than once"
			self assert: (expectedResult class = reflectiveResult class).
	]! !

!EventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 15:12'!
testObjectFor: aMethod
	|testObj|
	testObj := nil.
	"these do not respond to new"
	aMethod methodClass = SmalltalkImage ifTrue: [ testObj := SmalltalkImage current ].
	aMethod methodClass = SmallInteger ifTrue: [ testObj := 3 ]. 
	aMethod methodClass = Symbol ifTrue: [ testObj := #meh ].
	"proto-object does not know perform:withEnoughArgs"
	aMethod methodClass = ProtoObject ifTrue: [ testObj := Object new ]. 
	aMethod methodClass isMeta ifTrue: [ testObj := aMethod methodClass soleInstance ].
	testObj ifNil: [ testObj := aMethod methodClass new ]. 
	
	^ testObj! !

EventTest subclass: #GlobalReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!GlobalReadTest commentStamp: '<historical>' prior: 0!
A GlobalReadTest is a test class for testing the behavior of GlobalRead!

!GlobalReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 14:08'!
event

	^ GlobalRead new! !

!GlobalReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 14:32'!
methods

	^ {DummyEventTestingClass>>#globalRead}! !

!GlobalReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:11'!
newLinkWith: aBlock

	"example of mix explicit and lazy reification (all three context / class / object reifications are found at runtime)
	but name is passed by the re-compilation process -- note here that name is known at installation time so can be hardwired
	instead of passed"

	^ MetaLink new
		metaObject: [:name | aBlock value. (thisContext sender receiver class bindingOf: name) value  ];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #IVReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!IVReadTest commentStamp: '<historical>' prior: 0!
An IVReadTest is a test class for testing the behavior of IVRead!

!IVReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 14:58'!
event

	^ IVRead new! !

!IVReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:01'!
methods

	^ { DummyEventTestingClass>>#readIV }! !

!IVReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:09'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name | aBlock value. thisContext sender receiver instVarNamed: name  ];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #IVWriteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!IVWriteTest commentStamp: '<historical>' prior: 0!
An IVWriteTest is a test class for testing the behavior of IVWrite!

!IVWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:02'!
event
	
	^ IVWrite new! !

!IVWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:03'!
methods

	^ { DummyEventTestingClass>>#writeIV }! !

!IVWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:14'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name :newValue | aBlock value. thisContext sender receiver instVarNamed: name put: newValue  ];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #LiteralReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!LiteralReadTest commentStamp: '<historical>' prior: 0!
A LiteralReadTest is a test class for testing the behavior of LiteralRead!

!LiteralReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:04'!
event
	^ LiteralRead new! !

!LiteralReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:05'!
methods

	^ { DummyEventTestingClass>>#literalRead }! !

!LiteralReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:35'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:node | aBlock value. node value  ];
		selector: #value:;
		control: #instead;
		arguments: #(node);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #MSGReceiveTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!MSGReceiveTest commentStamp: '<historical>' prior: 0!
A MSGReceiveTest is a test class for testing the behavior of MSGReceive!

!MSGReceiveTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:07'!
event

	^ MSGReceive new! !

!MSGReceiveTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:07'!
methods

	^ { DummyEventTestingClass>>#msgReceive }! !

!MSGReceiveTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:42'!
newLinkWith: aBlock

		|initialMethod|

	initialMethod := self methods first.

	^ MetaLink new
		metaObject: [
			aBlock value. 
			Transcript show: 'Invoking.. intercessed';cr; flush.
			initialMethod  valueWithReceiver: thisContext sender receiver arguments: thisContext sender arguments. 
		];
		selector: #value;
		control: #instead;
		"arguments: #(node link);"
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #MSGSendTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!MSGSendTest commentStamp: '<historical>' prior: 0!
A MSGSendTest is a test class for testing the behavior of MSGSend!

!MSGSendTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:08'!
event

	^ MSGSend new! !

!MSGSendTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:19'!
methods

	^ { DummyEventTestingClass>>#msgSend . DummyEventTestingClass>>#aMethod }! !

!MSGSendTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 15:19'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:receiver :selector :arguments | aBlock value. receiver perform: selector withArguments: arguments  ];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(receiver selector arguments);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #MethodExecutionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!MethodExecutionTest commentStamp: '<historical>' prior: 0!
A MethodExecutionTest is a test class for testing the behavior of MethodExecution!

!MethodExecutionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:56'!
event

	^ MethodExecution new! !

!MethodExecutionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 15:58'!
methods

	^ { DummyEventTestingClass>>#methodExecution }! !

!MethodExecutionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:33'!
newLinkWith: aBlock

	|initialMethod|

	initialMethod := self methods first.

	^ MetaLink new
		metaObject: [
			aBlock value. 
			Transcript show: 'Invoking.. intercessed';cr; flush.
			initialMethod  valueWithReceiver: thisContext sender receiver arguments: thisContext sender arguments. 
		];
		selector: #value;
		control: #instead;
		"arguments: #(node link);"
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #ObjectCreationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!ObjectCreationTest commentStamp: '<historical>' prior: 0!
An ObjectCreationTest is a test class for testing the behavior of ObjectCreation!

!ObjectCreationTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:02'!
event

	^ ObjectCreation new! !

!ObjectCreationTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 14:35'!
methods

	^ { DummyEventTestingClass class>>#new }! !

!ObjectCreationTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:34'!
newLinkWith: aBlock

		|initialMethod|

	initialMethod := self methods first.

	^ MetaLink new
		metaObject: [
			aBlock value. 
			Transcript show: 'Invoking.. intercessed';cr; flush.
			initialMethod  valueWithReceiver: thisContext sender receiver arguments: thisContext sender arguments. 
		];
		selector: #value;
		control: #instead;
		"arguments: #(node link);"
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #PassedAsArgTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!PassedAsArgTest commentStamp: '<historical>' prior: 0!
A PassedAsArgTest is a test class for testing the behavior of PassedAsArg!

!PassedAsArgTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 17:02'!
event

	^ PassedAsArg new! !

!PassedAsArgTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/6/2015 11:01'!
methods

	^ { DummyEventTestingClass>>#passedAsArg }! !

!PassedAsArgTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:11'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:receiver :selector :arguments | aBlock value. 
			"normally checking the args here for the one you are looking"

			"Transcript show: receiver; cr; show: selector; cr; show: arguments; cr."
			thisContext sender top perform: selector withArguments: arguments ];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(receiver selector arguments);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #ReturnTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!ReturnTest commentStamp: '<historical>' prior: 0!
A ReturnTest is a test class for testing the behavior of Return!

!ReturnTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 17:00'!
event

	^ Return new! !

!ReturnTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 17:01'!
methods

	^ { DummyEventTestingClass>>#return }! !

!ReturnTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/9/2015 15:33'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:theNode :theLink | 
			|context initialMethod return|
			context := thisContext sender.
			aBlock value .
			theLink uninstall.
			initialMethod := context method methodClass >> context method selector.  
			return := initialMethod  valueWithReceiver: context receiver arguments: context arguments.  
			theNode link: theLink.
			return.
		];
		selector: #value:value:;
		control: #instead;
		arguments: #(node link);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #ReturnedTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!ReturnedTest commentStamp: '<historical>' prior: 0!
A ReturnedTest is a test class for testing the behavior of Returned!

!ReturnedTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 17:00'!
event

	^ Return new! !

!ReturnedTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 17:00'!
methods

	^ { DummyEventTestingClass>>#returned: }! !

!ReturnedTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/9/2015 15:32'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:theNode :theLink | 
			|context initialMethod return|
			context := thisContext sender.
			aBlock value .
			theLink uninstall.
			initialMethod := context method methodClass >> context method selector.  
			return := initialMethod  valueWithReceiver: context receiver arguments: context arguments.  
			theNode link: theLink.
			return.
		];
		selector: #value:value:;
		control: #instead;
		arguments: #(node link);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #StoredTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!StoredTest commentStamp: '<historical>' prior: 0!
A StoredTest is a test class for testing the behavior of Stored!

!StoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:59'!
event
	
	^ Stored new! !

!StoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:59'!
methods

	^ { DummyEventTestingClass>>#stored: } ! !

!StoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 16:34'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:node :name :newValue | aBlock value. 
			node variable class = RBInstanceVariableNode "this is where we will check for the obj
			in question"
				ifTrue: [thisContext sender receiver instVarNamed: name put: newValue]  
				ifFalse: [ thisContext sender tempNamed: name put: newValue ] "temporary"
		];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(node name newValue);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #TempReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!TempReadTest commentStamp: '<historical>' prior: 0!
A TempReadTest is a test class for testing the behavior of TempRead!

!TempReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:57'!
event

	^ TempRead new! !

!TempReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:57'!
methods

	^ { DummyEventTestingClass>>#tempRead }! !

!TempReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 17:11'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name | aBlock value. thisContext sender tempNamed: name  ];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).! !

EventTest subclass: #TempWriteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests-Vanilla'!

!TempWriteTest commentStamp: '<historical>' prior: 0!
A TempStoreTest is a test class for testing the behavior of TempStore!

!TempWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:42'!
event
	^ TempWrite new! !

!TempWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 16:56'!
methods

	^ { DummyEventTestingClass>>#tempWrite }! !

!TempWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 16:34'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name :newValue | aBlock value. thisContext sender tempNamed: name put: newValue  ];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).! !

TestCase subclass: #ProcessEventTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Context-Process-Add-Ons'!

!ProcessEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:07'!
testAProcess

	|targetProcess targetContext conditionCounter actionCounter shouldNotImplement|

	targetProcess := [DummyEventTestingClass new aMethodWithSuspendedContext] fork.
	(Delay forSeconds: 2) wait.
	targetContext := targetProcess suspendedContext.

	conditionCounter := 0.
	actionCounter := 0.  
	shouldNotImplement := false. 

	[ReflectoGram 
		on: targetProcess
		as: Entity -> Flat
		for: MSGSend 
		limitedTo: [ :method | true ]
		when: [ :rPoint | conditionCounter := conditionCounter + 1 . true ]
		do: [ :rPoint |  actionCounter := actionCounter + 1. rPoint returnValue: rPoint defaultAction value]] 
						on: ShouldNotImplement do: [ 
							Transcript show: 'DNI: '; show: targetContext class; show: '->'; show: self class; cr. shouldNotImplement := true ].  
						
	targetProcess resume.
	
	(Delay forSeconds: 2) wait.
	
	self assert: (conditionCounter = 2).
	self assert: (actionCounter = 2)! !

TestCase subclass: #TargetEventTest
	instanceVariableNames: 'link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetEventTest class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:34'!
isAbstract

	^self name = #TargetEventTest! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:16'!
checkMethod: aMethod
	
	|firstMethod|
	
	firstMethod := self methods first.

	^ (aMethod selector = firstMethod selector) and: [ aMethod methodClass name = firstMethod methodClass name ] 
! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:42'!
event

	^ Event new
! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:09'!
expectedResultFor: aMethod

	|testObj result|
	testObj := self testObjectFor: aMethod.
	"[" result := testObj perform: aMethod selector withEnoughArguments: { }."]"
		"on: Exception 
		do: [:ex | result := ex ]."
	
	^ result! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:09'!
expectedResultFor: aMethod usingTestObject: aTestObject

	| result|
	"[" result := aTestObject perform: aMethod selector withEnoughArguments: { }."]"
		"on: Exception 
		do: [:ex | result := ex ]."
	
	^ result! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:42'!
methods

	^ { }! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:42'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: aBlock;
		selector: #value;
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:43'!
targets

	^ { Smalltalk . "image" 
	self class package . "package"
	DummyEventTestingClass . "class"
	DummyEventTestingClass>>#aMethod . "method" 
	DummyEventTestingClass new . "object"}
	"[ DummyEventTestingClass new aMethodWithSuspendedContext ] newProcess . "process"
	[ DummyEventTestingClass new aMethodWithSuspendedContext ] newProcess resume suspendedContext "context" }"! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 11:43'!
tearDown

	"additonal clean up"	
	
	link ifNotNilDo: [:l | l uninstall].
	self methods do: [:method | method recompile].	 "clean up and revert method to match each source-code"
	DummyEventTestingClass recompile.
	super tearDown.! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:42'!
testEvent

	| expectedResult reflectiveResult metaJumpOccured |
	self methods do: [:method |
		expectedResult := self expectedResultFor: method.
		link := self newLinkWith: [metaJumpOccured := true].  
		self event 
			annotateMethod: method
			withLink: link.
			reflectiveResult := self expectedResultFor: method.
			link uninstall.
			method recompile.	"clean up and revert method to match each source-code"
			self assert: metaJumpOccured = true. "some events like sends may be triggered more than once"
			self assert: (expectedResult class = reflectiveResult class).
	]! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 17:31'!
testEventFor: target

	|conditionCounter actionCounter reflectiveResult expectedResult reflectoGram shouldNotImplement|

			conditionCounter := 0.
			actionCounter := 0.  
			shouldNotImplement := false. 
			target class = CompiledMethod 
				ifTrue: [expectedResult := self expectedResultFor: target. ]
				ifFalse: [ expectedResult := self expectedResultFor: self methods first. ].
			[reflectoGram := (ReflectoGram 
				on: target 
				as: Entity -> Flat
				for: self event 
				limitedTo: [ :method | (target class = CompiledMethod) 
												ifFalse: [self checkMethod: method]
												ifTrue: [ true ] ] 
				when: [ :rPoint | conditionCounter := conditionCounter + 1 . true ]
				do: [ :rPoint |  actionCounter := actionCounter + 1. rPoint returnValue: rPoint defaultAction value]) enable] 
						on: ShouldNotImplement do: [ 
							Transcript show: 'DNI: '; show: target class; show: '->'; show: self class; cr. shouldNotImplement := true ].
			shouldNotImplement ifFalse: [  
				self assert: conditionCounter = 0.
				self assert: actionCounter = 0.
				target class = DummyEventTestingClass "vanilla object target use it directly for testing" 
					ifTrue: [ reflectiveResult := self expectedResultFor: self methods first usingTestObject: target ]
					ifFalse: [ 
						target class = CompiledMethod 
							ifTrue: [ reflectiveResult := self expectedResultFor: target. ]
							ifFalse: [ reflectiveResult := self expectedResultFor: self methods first. ]. 
					].
				reflectoGram uninstall.
				self methods first recompile.	"clean up and revert method to match each source-code"
				target class = CompiledMethod ifTrue: [ target recompile ].
				self assert: conditionCounter >= 1. "aMethod example has multiple events - vanilla test check the strict case to test recursion problems"
				self assert: actionCounter >= 1.
				self assert: conditionCounter >= 1.
				self assert: actionCounter >= 1.
				self assert: (expectedResult class = reflectiveResult class).
			]
		! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:07'!
testEventForClassTarget

	self testEventFor: DummyEventTestingClass. ! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:04'!
testEventForImageTarget

    self testEventFor: Smalltalk.! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:38'!
testEventForMethodTarget

	"(DummyEventTestingClass>>#aMethod) recompile."
	self testEventFor: self methods first.! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:05'!
testEventForObjectTarget

	self testEventFor: DummyEventTestingClass new . "object"! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:16'!
testEventForPackageTarget

	self testEventFor: self class package . "package"! !

!TargetEventTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:42'!
testObjectFor: aMethod
	|testObj|
	testObj := nil.
	"these do not respond to new"
	aMethod methodClass = SmalltalkImage ifTrue: [ testObj := SmalltalkImage current ].
	aMethod methodClass = SmallInteger ifTrue: [ testObj := 3 ]. 
	aMethod methodClass = Symbol ifTrue: [ testObj := #meh ].
	"proto-object does not know perform:withEnoughArgs"
	aMethod methodClass = ProtoObject ifTrue: [ testObj := Object new ]. 
	aMethod methodClass isMeta ifTrue: [ testObj := aMethod methodClass soleInstance ].
	testObj ifNil: [ testObj := aMethod methodClass new ]. 
	
	^ testObj! !

TargetEventTest subclass: #TargetGlobalReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetGlobalReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:45'!
event

	^ GlobalRead new! !

!TargetGlobalReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:49'!
methods

	^ {DummyEventTestingClass>>#globalRead}! !

!TargetGlobalReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 12:45'!
newLinkWith: aBlock

	"example of mix explicit and lazy reification (all three context / class / object reifications are found at runtime)
	but name is passed by the re-compilation process -- note here that name is known at installation time so can be hardwired
	instead of passed"

	^ MetaLink new
		metaObject: [:name | aBlock value. (thisContext sender receiver class bindingOf: name) value  ];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetIVReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetIVReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:33'!
event

	^ IVRead new! !

!TargetIVReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:33'!
methods

	^ { DummyEventTestingClass>>#readIV }! !

!TargetIVReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:33'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name | aBlock value. thisContext sender receiver instVarNamed: name  ];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetIVWriteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetIVWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
event
	
	^ IVWrite new! !

!TargetIVWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
methods

	^ { DummyEventTestingClass>>#writeIV }! !

!TargetIVWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name :newValue | aBlock value. thisContext sender receiver instVarNamed: name put: newValue  ];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetLiteralReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetLiteralReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
event
	^ LiteralRead new! !

!TargetLiteralReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
methods

	^ { DummyEventTestingClass>>#literalRead }! !

!TargetLiteralReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:node | aBlock value. node value  ];
		selector: #value:;
		control: #instead;
		arguments: #(node);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetMSGReceiveTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetMSGReceiveTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
event

	^ MSGReceive new! !

!TargetMSGReceiveTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:34'!
methods

	^ { DummyEventTestingClass>>#msgReceive }! !

!TargetMSGReceiveTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:44'!
newLinkWith: aBlock

		|initialMethod|

	initialMethod := self methods first.

	^ MetaLink new
		metaObject: [
			aBlock value. 
			Transcript show: 'Invoking.. intercessed';cr; flush.
			initialMethod  valueWithReceiver: thisContext sender receiver arguments: thisContext sender arguments. 
		];
		selector: #value;
		control: #instead;
		"arguments: #(node link);"
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetMSGSendTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetMSGSendTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
event

	^ MSGSend new! !

!TargetMSGSendTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
methods

	^ { DummyEventTestingClass>>#msgSend }! !

!TargetMSGSendTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:receiver :selector :arguments | aBlock value. receiver perform: selector withArguments: arguments  ];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(receiver selector arguments);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetMethodExecutionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetMethodExecutionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
event

	^ MethodExecution new! !

!TargetMethodExecutionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
methods

	^ { DummyEventTestingClass>>#methodExecution }! !

!TargetMethodExecutionTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:44'!
newLinkWith: aBlock

		|initialMethod|

	initialMethod := self methods first.

	^ MetaLink new
		metaObject: [
			aBlock value. 
			Transcript show: 'Invoking.. intercessed';cr; flush.
			initialMethod  valueWithReceiver: thisContext sender receiver arguments: thisContext sender arguments. 
		];
		selector: #value;
		control: #instead;
		"arguments: #(node link);"
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetObjectCreationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetObjectCreationTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
event

	^ ObjectCreation new! !

!TargetObjectCreationTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:37'!
methods

	^ { DummyEventTestingClass class>>#new }! !

!TargetObjectCreationTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:44'!
newLinkWith: aBlock

		|initialMethod|

	initialMethod := self methods first.

	^ MetaLink new
		metaObject: [
			aBlock value. 
			Transcript show: 'Invoking.. intercessed';cr; flush.
			initialMethod  valueWithReceiver: thisContext sender receiver arguments: thisContext sender arguments. 
		];
		selector: #value;
		control: #instead;
		"arguments: #(node link);"
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetPassedAsArgTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetPassedAsArgTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/13/2015 12:39'!
event

	^ PassedAsArg new! !

!TargetPassedAsArgTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/13/2015 12:39'!
methods

	^ { DummyEventTestingClass>>#passedAsArg }! !

!TargetPassedAsArgTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 16:11'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:receiver :selector :arguments | aBlock value. 
			"normally checking the args here for the one you are looking"

			"Transcript show: receiver; cr; show: selector; cr; show: arguments; cr."
			thisContext sender top perform: selector withArguments: arguments ];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(receiver selector arguments);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetReturnTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetReturnTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
event

	^ Return new! !

!TargetReturnTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
methods

	^ { DummyEventTestingClass>>#return }! !

!TargetReturnTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:theNode :theLink | 
			|context initialMethod return|
			context := thisContext sender.
			aBlock value .
			theLink uninstall.
			initialMethod := context method methodClass >> context method selector.  
			return := initialMethod  valueWithReceiver: context receiver arguments: context arguments.  
			theNode link: theLink.
			return.
		];
		selector: #value:value:;
		control: #instead;
		arguments: #(node link);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetReturnedTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetReturnedTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
event

	^ Return new! !

!TargetReturnedTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
methods

	^ { DummyEventTestingClass>>#returned: }! !

!TargetReturnedTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:theNode :theLink | 
			|context initialMethod return|
			context := thisContext sender.
			aBlock value .
			theLink uninstall.
			initialMethod := context method methodClass >> context method selector.  
			return := initialMethod  valueWithReceiver: context receiver arguments: context arguments.  
			theNode link: theLink.
			return.
		];
		selector: #value:value:;
		control: #instead;
		arguments: #(node link);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetStoredTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetStoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
event
	
	^ Stored new! !

!TargetStoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:38'!
expectedResultFor: aMethod

	|testObj result|
	testObj := self testObjectFor: aMethod.
	"[" result := testObj perform: aMethod selector withEnoughArguments: { }."]"
		"on: Exception 
		do: [:ex | result := ex ]."
	
	^ result! !

!TargetStoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:45'!
expectedResultFor: aMethod usingTestObject: aTestObject

	| result testReceiver|
	testReceiver := self testObjectFor: aMethod.
	aTestObject class = DummyEventTestingClass 
		ifTrue: [ result := testReceiver perform: aMethod selector withArguments: {aTestObject}  ]
		ifFalse: [ result := testReceiver perform: aMethod selector withArguments: {testReceiver} ]
	"[" ."]"
		"on: Exception 
		do: [:ex | result := ex ]."
	
	^ result! !

!TargetStoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
methods

	^ { DummyEventTestingClass>>#stored: } ! !

!TargetStoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:node :name :newValue | aBlock value. 
			node variable class = RBInstanceVariableNode "this is where we will check for the obj
			in question"
				ifTrue: [thisContext sender receiver instVarNamed: name put: newValue]  
				ifFalse: [ thisContext sender tempNamed: name put: newValue ] "temporary"
		];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(node name newValue);
		options: #(+ optionCompileOnLinkInstallation).! !

!TargetStoredTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:52'!
testEventFor: target

	|conditionCounter actionCounter reflectiveResult expectedResult reflectoGram shouldNotImplement|

			conditionCounter := 0.
			actionCounter := 0.  
			shouldNotImplement := false. 
			target class = CompiledMethod 
				ifTrue: [expectedResult := self expectedResultFor: target. ]
				ifFalse: [ expectedResult := self expectedResultFor: self methods first usingTestObject: target. ].
			[reflectoGram := ReflectoGram 
				on: target 
				as: Entity -> Flat
				for: self event 
				limitedTo: [ :method | (target class = CompiledMethod) 
												ifFalse: [self checkMethod: method]
												ifTrue: [ true ] ] 
				when: [ :rPoint | conditionCounter := conditionCounter + 1 . true ]
				do: [ :rPoint |  actionCounter := actionCounter + 1. rPoint returnValue: rPoint defaultAction value]] 
						on: ShouldNotImplement do: [ 
							Transcript show: 'DNI: '; show: target class; show: '->'; show: self class; cr. shouldNotImplement := true ].
			shouldNotImplement ifFalse: [  
				self assert: conditionCounter = 0.
				self assert: actionCounter = 0.
				target class = DummyEventTestingClass "vanilla object target use it directly for testing" 
					ifTrue: [ reflectiveResult := self expectedResultFor: self methods first usingTestObject: target ]
					ifFalse: [ 
						target class = CompiledMethod 
							ifTrue: [ reflectiveResult := self expectedResultFor: target. ]
							ifFalse: [ reflectiveResult := self expectedResultFor: self methods first usingTestObject: target ]. 
					].
				reflectoGram uninstall.
				self methods first recompile.	"clean up and revert method to match each source-code"
				target class = CompiledMethod ifTrue: [ target recompile ].
				self assert: conditionCounter >= 1. "aMethod example has multiple events - vanilla test check the strict case to test recursion problems"
				self assert: actionCounter >= 1.
				self assert: conditionCounter >= 1.
				self assert: actionCounter >= 1.
				self assert: (expectedResult class = reflectiveResult class).
			]
		! !

TargetEventTest subclass: #TargetTempReadTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetTempReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
event

	^ TempRead new! !

!TargetTempReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
methods

	^ { DummyEventTestingClass>>#tempRead }! !

!TargetTempReadTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name | aBlock value. thisContext sender tempNamed: name  ];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).! !

TargetEventTest subclass: #TargetTempWriteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!TargetTempWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
event
	^ TempWrite new! !

!TargetTempWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
methods

	^ { DummyEventTestingClass>>#tempWrite }! !

!TargetTempWriteTest methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 15:38'!
newLinkWith: aBlock

	^ MetaLink new
		metaObject: [:name :newValue | aBlock value. thisContext sender tempNamed: name put: newValue  ];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).! !

Object subclass: #BenchmarkObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Benchmarks'!

!BenchmarkObject methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/10/2013 14:41'!
test
	| a b c |
	a := 1.
	b := 2.
	c := a + b.
	^ c
! !

Object subclass: #DummyEventTestingClass
	instanceVariableNames: 'anIv'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Tests'!

!DummyEventTestingClass class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 14:35'!
new

	"intresting here that you should either add new if it does not exist
	or intercess it in the superclass using a framework condition"

	^ super new! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:20'!
aMethod
	|x t|
	
	self lorem ipsum bar.
	t := 3.
	t + 2.
	x := anIv.
	anIv := Object. 
	Transcript show: Object new.
	^ Object.! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:03'!
aMethodWithSuspendedContext

	3 + 4.
	
	Processor activeProcess suspend.
	
	self doSomeSimpleComputation. "does smth"! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/12/2015 16:28'!
aProcess! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:45'!
bar

	^ 3! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:05'!
doSomeSimpleComputation

	3 + 4.! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:04'!
foo! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:07'!
foo: anArgument! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:54'!
globalRead

	^ Object! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:15'!
initialize
	

	anIv = 42! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:04'!
ipsum! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:57'!
literalRead

	^ 3! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:03'!
lorem! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:33'!
methodExecution

	self lorem ipsum foo bar.
	Transcript show: 'I am executing...before (expected results) intercession and after intercession (reflective result)';cr.
	^ 3! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:45'!
msgReceive

	^ self lorem ipsum foo bar! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 13:01'!
msgSend

	^ self lorem ipsum foo bar! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 14:33'!
objectCreation

	"the new method on the other side will be intercessed"! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/16/2015 19:33'!
passedAsArg

	^ self foo: self! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 19:59'!
readIV
	|x|
	x := anIv.
	^ Object ! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:07'!
return

	^ 3! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:08'!
returned: anIntercessedObject

	^ anIntercessedObject ! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:08'!
stored: anIntercessedObject

	anIv := anIntercessedObject ! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 17:12'!
tempRead

	|t|
	
	t := 3.
	^ t + 2! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/4/2015 11:10'!
tempWrite

	|t|
	
	t := 3.
	t + 2! !

!DummyEventTestingClass methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:15'!
writeIV

	^ anIv := 3 ! !

Object subclass: #Event
	uses: TRBProgramNodeVisitor
	instanceVariableNames: 'link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

Event class
	uses: TRBProgramNodeVisitor classTrait
	instanceVariableNames: 'default'!

Event class
	uses: TRBProgramNodeVisitor classTrait
	instanceVariableNames: 'default'!

!Event methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/15/2015 16:10'!
annotateMethod: aMethod forTarget: aTarget 

	self subclassResponsibility 
! !

!Event methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 14:35'!
annotateMethod: aMethod withLink: aMetaLink 

	link := aMetaLink.
	self visitNode: aMethod ast! !

!Event methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 15:36'!
resolveScopeFor: aTarget

	self subclassResponsibility ! !

!Event methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/23/2015 20:08'!
scope

	self subclassResponsibility ! !

!Event methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 16:56'!
uninstall

	link ifNotNil: [ link uninstall ]! !

Event subclass: #MultiScopedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!MultiScopedEvent methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:34'!
scope

	^ MultiScope! !

MultiScopedEvent subclass: #RealMultiScoped
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

RealMultiScoped subclass: #PassedAsArg
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!PassedAsArg methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 12:54'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:receiver :selector :arguments | 
			Interpreter 
				target: aTarget 
				passedAsArgWith: thisContext sender top "this is a hack to fix bug with receiver on reflectivity" 
				and: selector 
				with: arguments];
		selector: #value:value:value:;
		control: #instead;
		arguments: #(receiver selector arguments);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!PassedAsArg methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/23/2015 20:04'!
visitMessageNode: aMessageNode 
	aMessageNode link: link.
	^ super visitMessageNode: aMessageNode.! !

!RealMultiScoped methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:01'!
resolveScopeFor: aTarget

	aTarget realMultiScopedEvent: self! !

!RealMultiScoped methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 15:59'!
scope

	^ RealMultiScope ! !

RealMultiScoped subclass: #Returned
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!Returned methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/9/2015 14:26'!
visitMethodNode: aMethodNode 
	aMethodNode link: link.
	^ super visitMethodNode: aMethodNode.! !

RealMultiScoped subclass: #Stored
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!Stored methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 13:08'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:name :newValue | Interpreter target: aTarget iv: name stored: newValue];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!Stored methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 13:04'!
visitAssignmentNode: anAssignmentNode
	"((anAssignmentNode variable isInstance))  
		ifTrue: [  ]."
	((anAssignmentNode variable isInstance)) "only stored in ivs" 
		ifTrue: [ anAssignmentNode link: link ].
	^ super visitAssignmentNode: anAssignmentNode.! !

MultiScopedEvent subclass: #SimulatedMultiScoped
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

SimulatedMultiScoped subclass: #MSGReceive
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!MSGReceive methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:46'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink|

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [ Interpreter target: aTarget msgReceiveForMethod: aMethod]; "this is the initialMethod that can be directly executed"
		selector: #value;
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!MSGReceive methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 15:21'!
visitMethodNode: aMethodNode 
	aMethodNode link: link.
	^ super visitMethodNode: aMethodNode.! !

SimulatedMultiScoped subclass: #ObjectCreation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!ObjectCreation methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:03'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink|

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [ Interpreter target: aTarget objectCreationFromMethod: aMethod]; "this is the initialMethod that can be directly executed"
		selector: #value;
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!ObjectCreation methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/11/2015 18:33'!
visitMethodNode: aMethodNode 
	aMethodNode link: link.
	^ super visitMethodNode: aMethodNode.! !

!SimulatedMultiScoped methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:01'!
resolveScopeFor: aTarget

	aTarget simulatedMultiScopedEvent: self! !

!SimulatedMultiScoped methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 15:59'!
scope

	^ SimulatedMultiScope ! !

Event subclass: #SingleScopedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

SingleScopedEvent subclass: #GlobalRead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!GlobalRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 14:42'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:name | Interpreter target: aTarget globalReadNamed: name];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!GlobalRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/5/2015 14:35'!
visitGlobalNode: aGlobalNode
	aGlobalNode link: link.
	^ super visitVariableNode: aGlobalNode.! !

SingleScopedEvent subclass: #IVRead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!IVRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 19:52'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:name | Interpreter target: aTarget ivRead: name];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!IVRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/6/2015 11:30'!
visitVariableNode: aVariableNode
	((aVariableNode isInstance) & (aVariableNode parent ~= AssignmentNode)) "read" 
		ifTrue: [ aVariableNode link: link ].
	^ super visitVariableNode: aVariableNode.! !

SingleScopedEvent subclass: #IVWrite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!IVWrite methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/18/2015 12:05'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:name :newValue | Interpreter target: aTarget iv: name write: newValue];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!IVWrite methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/6/2015 11:32'!
visitAssignmentNode: anAssignmentNode
	((anAssignmentNode variable isInstance)) "iv" 
		ifTrue: [ anAssignmentNode link: link ].
	^ super visitAssignmentNode: anAssignmentNode.! !

SingleScopedEvent subclass: #LiteralRead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!LiteralRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/18/2015 12:21'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:node | Interpreter target: aTarget literalRead: node value];
		selector: #value:;
		control: #instead;
		arguments: #(node);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!LiteralRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/7/2015 12:32'!
visitLiteralNode: aLiteralNode

	aLiteralNode link: link.
	^ super visitVariableNode: aLiteralNode.! !

SingleScopedEvent subclass: #MSGSend
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!MSGSend methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:47'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:receiver :selector :arguments | Interpreter target: aTarget msgSendWith: receiver "this is a hack to fix bug with receiver on reflectivity"  and: selector with: arguments];
		selector: #value:value:value:;
		control: #instead; 
		arguments: #(receiver selector arguments);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!MSGSend methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/23/2015 18:14'!
visitMessageNode: aMessageNode 
	aMessageNode link: link.
	^ super visitMessageNode: aMessageNode.! !

SingleScopedEvent subclass: #MethodExecution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!MethodExecution methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:28'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink|

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [ Interpreter target: aTarget executionForMethod: aMethod]; "this is the initialMethod that can be directly executed"
		selector: #value;
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!MethodExecution methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:32'!
visitMethodNode: aMethodNode 

	aMethodNode link: link.
	^ super visitMethodNode: aMethodNode.! !

SingleScopedEvent subclass: #Return
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!Return methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 09:25'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:theNode :theLink | "after return"
			|context initialMethod return|
			context := thisContext sender.
			theLink uninstall.
			initialMethod := context method methodClass >> context method selector.  
			return := initialMethod  valueWithReceiver: context receiver arguments: context arguments.  
			theNode link: theLink.
			Interpreter target: aTarget return: return
		];
		selector: #value:value:;
		control: #instead;
		arguments: #(node link);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!Return methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/9/2015 14:26'!
visitMethodNode: aMethodNode 
	aMethodNode link: link.
	^ super visitMethodNode: aMethodNode.! !

!SingleScopedEvent methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 15:36'!
resolveScopeFor: aTarget

	aTarget singleScopedEvent: self.! !

!SingleScopedEvent methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:34'!
scope

	^ SingleScope! !

SingleScopedEvent subclass: #TempRead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!TempRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 09:36'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:name | Interpreter target: aTarget tempRead: name];
		selector: #value:;
		control: #instead;
		arguments: #(name);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!TempRead methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/9/2015 14:09'!
visitTemporaryNode: aNode 
	"Sent *each time* a temporary node is found"
	aNode isRead "read" 
		ifTrue: [ aNode link: link ].
	^ super visitTemporaryNode: aNode.! !

SingleScopedEvent subclass: #TempWrite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Events'!

!TempWrite methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 09:39'!
annotateMethod: aMethod forTarget: aTarget 
	
	|aLink |

	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:name :newValue | Interpreter target: aTarget temp: name write: newValue];
		selector: #value:value:;
		control: #instead;
		arguments: #(name newValue);
		options: #(+ optionCompileOnLinkInstallation).
		self annotateMethod: aMethod withLink: aLink.! !

!TempWrite methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/6/2015 12:23'!
visitAssignmentNode: anAssignmentNode
	((anAssignmentNode variable isTemp)) "iv" 
		ifTrue: [ anAssignmentNode link: link ].
	^ super visitAssignmentNode: anAssignmentNode.! !

!TempWrite methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/6/2015 11:34'!
visitStoreIntoTempNode: aNode
	"this is never called -- is done through assignment visit and checking"
	aNode link: link.
	^ super visitStoreIntoTempNode: aNode.! !

Object subclass: #Interpreter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Interpreter-MObjects'!

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 17:23'!
metaJumpForTarget: aTarget withDefaultAction: defaultAction

	|p returnValue condition action|

	condition := aTarget userCondition.
	action := aTarget userAction.
	aTarget disable.

	p := aTarget reflectogram 
					newPointFor: aTarget
					withDefaultAction: defaultAction.

	condition ifNil: [  
		action value: p
	] ifNotNil: [ 
		(condition value: p) 
			ifTrue: [ action value: p ]
			ifFalse: [ returnValue := defaultAction value ].
	]. 

	p override ifTrue: [ returnValue :=  p returnValue] ifFalse: [ returnValue := defaultAction value ].
	
	aTarget enable.
		
	^ returnValue.! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:32'!
target: aTarget executionForMethod: aMethod
				
		|defaultAction eventReceiver arguments|
		
		eventReceiver := thisContext sender sender receiver.
		arguments := thisContext sender sender arguments.
		
		defaultAction := [aMethod valueWithReceiver: eventReceiver arguments: arguments].

		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:29'!
target: aTarget globalReadNamed: aName

		|defaultAction eventClass eventReceiver|
		eventReceiver := thisContext sender sender receiver. 
		eventClass := eventReceiver class.
		defaultAction := [(eventClass bindingOf: aName) value].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:55'!
target: aTarget iv: aName stored: newValue

		|defaultAction eventReceiver|

		eventReceiver := thisContext sender sender receiver.
		defaultAction := [eventReceiver instVarNamed: aName put: newValue].

		^ (aTarget entityConditionGroupingInstances value: newValue)
			 ifTrue: [self metaJumpForTarget: aTarget withDefaultAction: defaultAction]
			 ifFalse: [ defaultAction value ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:29'!
target: aTarget iv: aName write: newValue

		|defaultAction eventReceiver|
		eventReceiver := thisContext sender sender receiver.
		defaultAction := [eventReceiver instVarNamed: aName put: newValue].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:29'!
target: aTarget ivRead: aName

		|defaultAction eventReceiver|
		eventReceiver := thisContext sender sender receiver.
		defaultAction := [eventReceiver instVarNamed: aName].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:30'!
target: aTarget literalRead: aLiteralValue

		|defaultAction|
		defaultAction := [aLiteralValue].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 16:55'!
target: aTarget msgReceiveForMethod: aMethod
			
		|defaultAction eventReceiver arguments|
		
		eventReceiver := thisContext sender sender receiver.
		arguments := thisContext sender sender arguments.
		
		defaultAction := [aMethod valueWithReceiver: eventReceiver arguments: arguments].

		^ (aTarget entityConditionGroupingInstances value: eventReceiver)
			 ifTrue: [self metaJumpForTarget: aTarget withDefaultAction: defaultAction]
			 ifFalse: [ defaultAction value ]
			! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 15:45'!
target: aTarget msgSendWith: receiver and: selector with: arguments

		|defaultAction|
	
		defaultAction := [receiver perform: selector withArguments: arguments].
	
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:19'!
target: aTarget objectCreationFromMethod: aMethod
			
		|defaultAction|
		defaultAction := [aMethod valueWithReceiver: aMethod methodClass theNonMetaClass arguments: #()]. "this is new"

		^ self metaJumpForTarget: aTarget withDefaultAction: defaultAction! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/18/2015 17:01'!
target: aTarget passedAsArgWith: receiver and: selector with: arguments

		|defaultAction|
		"normally you can get all reifications lazily"
		defaultAction := [receiver perform: selector withArguments: arguments].
		^ (arguments isEmpty not and: [(arguments select: [:arg | aTarget entityConditionGroupingInstances value: arg]) isEmpty not])
				ifTrue: [  
						 self metaJumpForTarget: aTarget withDefaultAction: defaultAction
				] ifFalse: [  
						 defaultAction value
				]! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:30'!
target: aTarget return: aReturnValue

		|defaultAction |

		defaultAction := [aReturnValue].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:31'!
target: aTarget temp: aName write: newValue

		|defaultAction eventReceiver eventContext|
		eventContext := thisContext sender sender.
		eventReceiver := eventContext receiver.
		defaultAction := [eventContext tempNamed: aName put: newValue].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

!Interpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:31'!
target: aTarget tempRead: aName

		|defaultAction eventReceiver eventContext|
		eventContext := thisContext sender sender.
		eventReceiver := eventContext receiver.
		defaultAction := [eventContext tempNamed: aName].
		^ aTarget isRunTime 
			ifFalse: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
			ifTrue: [ (aTarget runTimeCondition)
								ifTrue: [ self metaJumpForTarget: aTarget withDefaultAction: defaultAction ]
								ifFalse: [ defaultAction value ] ]
			
	! !

Object subclass: #LowLevelImplicitMetaObject
	instanceVariableNames: 'condition action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Interpreter-MObjects'!

!LowLevelImplicitMetaObject methodsFor: 'accessing' stamp: 'NickPapoulias 11/17/2015 10:52'!
action
	^ action! !

!LowLevelImplicitMetaObject methodsFor: 'accessing' stamp: 'NickPapoulias 11/17/2015 10:52'!
action: anObject
	action := anObject! !

!LowLevelImplicitMetaObject methodsFor: 'accessing' stamp: 'NickPapoulias 11/17/2015 10:52'!
condition
	^ condition! !

!LowLevelImplicitMetaObject methodsFor: 'accessing' stamp: 'NickPapoulias 11/17/2015 10:52'!
condition: anObject
	condition := anObject! !

Object subclass: #MetaBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Benchmarks'!

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 12:13'!
benchmark: aBlock times: anInteger
	"synced with Bifrost's bench"
	| runs average deviation |
	aBlock value; value. "this does nothing in our case possible bifrost uses it
	to install methodWrappers the first time this is run ?"
	runs := [ 
		(1 to: anInteger) 
			collect: [ :each | Time millisecondsToRun: aBlock ] ] "equiv. to timeToRun in previous version which now returns duration"
		valueUnpreemptively.
	average := runs average asFloat.
	deviation := ((runs 
		detectSum: [ :each | (each - average) * (each - average) ]) 
			/ runs size) sqrt.
	^ Array with: average with: deviation! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:48'!
message
	"self new message" 
	
	|  object  |
	object := BenchmarkObject new.
	^ self benchmark: [ 1 to: 1000000 do: [ :each | self methodWithSendFor: object ] ] times: 100.! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:49'!
messageMetaSend
	"self new messageMetaSend"
	
	|  object result aLink|
	object := BenchmarkObject new.
	
	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:receiver :selector :arguments | receiver perform: selector withArguments: arguments ]; "this is the initialMethod that can be directly executed"
		selector: #value:value:value:;
		arguments: #(receiver selector arguments);
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).
	 
	(self class >> #methodWithSendFor:) ast sendNodes first link: aLink.
	
	self assert: ((self methodWithSendFor: object) = 3).
	
	result := self
		benchmark: [ 1 to: 1000000 do: [ :each | self methodWithSendFor: object ] ]
		times: 10.
	
	aLink uninstall. 
	(self class >> #methodWithSendFor:) recompile.
	
	^ result! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:55'!
messageMetaSendBefore
	"self new messageMetaSendBefore"
	
	|  object result aLink|
	object := BenchmarkObject new.
	
	aLink := MetaLink new "something like this should be optimized"
		metaObject: [ ]; "this is the initialMethod that can be directly executed"
		selector: #value;
		"arguments: #(receiver selector arguments);"
		control: #before;
		options: #(+ optionCompileOnLinkInstallation).
	 
	(self class >> #methodWithSendFor:) ast sendNodes first link: aLink.
	
	self assert: ((self methodWithSendFor: object) = 3).
	
	result := self
		benchmark: [ 1 to: 1000000 do: [ :each | self methodWithSendFor: object ] ]
		times: 10.
	
	aLink uninstall. 
	(self class >> #methodWithSendFor:) recompile.
	
	^ result! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:55'!
messageMetaSendInstead
	"self new messageMetaSendInstead"
	
	|  object result aLink|
	object := BenchmarkObject new.
	
	aLink := MetaLink new "something like this should be optimized"
		metaObject: [:receiver :selector :arguments | receiver perform: selector withArguments: arguments ]; "this is the initialMethod that can be directly executed"
		selector: #value:value:value:;
		arguments: #(receiver selector arguments);
		control: #instead;
		options: #(+ optionCompileOnLinkInstallation).
	 
	(self class >> #methodWithSendFor:) ast sendNodes first link: aLink.
	
	self assert: ((self methodWithSendFor: object) = 3).
	
	result := self
		benchmark: [ 1 to: 1000000 do: [ :each | self methodWithSendFor: object ] ]
		times: 10.
	
	aLink uninstall. 
	(self class >> #methodWithSendFor:) recompile.
	
	^ result! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:56'!
messageMetaSendWithReflectogram
	"self new messageMetaSendWithReflectogram" 
	
	|  object result reflectogram|
	object := BenchmarkObject new.
	
	reflectogram := ReflectoGram on: (self class >> #methodWithSendFor:)
			for: MSGSend  
			when: nil
			do: [ :rPoint | ].
	 	
	self assert: (self methodWithSendFor: object) = 3.
	
	result := self
		benchmark: [ 1 to: 1000000 do: [ :each |(self methodWithSendFor: object) = 3 ] ]
		times: 10.
	
	reflectogram uninstall remove.
	
	^ result! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 13:30'!
methodWithSendFor: anObject

	^ anObject test
	! !

!MetaBenchmark methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 17:23'!
stats

	| vanilla metaBefore metaInstead metaWithReflectogram metaBeforeDivVanilla metaInsteadDivVanilla metaWithReflectogramDivVanilla reflectogramDivMetaInstead |
	
	MetaBenchmark recompile.
	
	vanilla := MetaBenchmark new message.
	metaBefore := MetaBenchmark new messageMetaSendBefore.
	metaInstead := MetaBenchmark new messageMetaSendInstead.
	metaWithReflectogram := MetaBenchmark new messageMetaSendWithReflectogram.
	
	metaBeforeDivVanilla := (metaBefore at: 1) / (vanilla at: 1).
	metaInsteadDivVanilla := (metaInstead at: 1) / (vanilla at: 1).
	metaWithReflectogramDivVanilla := (metaWithReflectogram at: 1) / (vanilla at: 1).
	reflectogramDivMetaInstead := (metaWithReflectogram at: 1) / (metaInstead at: 1).
	
	Transcript 
		show: 'VANILLA: ' , vanilla asString; cr;
		show: 'META-BEFORE: ' , metaBefore asString; cr;
		show: 'META-INSTEAD: ' , metaInstead asString; cr;
		show: 'META-REFL: ' , metaWithReflectogram asString; cr;
		show: 'M-Before/V: ' , metaBeforeDivVanilla asString; cr;
		show: 'M-Instead/V: ' , metaInsteadDivVanilla asString; cr;
		show: 'M-Refl/V: ' , metaWithReflectogramDivVanilla asString; cr;
		show: 'M-Refl/M-Instead: ', reflectogramDivMetaInstead asString; cr.

! !

Object subclass: #Mode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Mode subclass: #Scope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Scope subclass: #MultiScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

MultiScope subclass: #RealMultiScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

MultiScope subclass: #SimulatedMultiScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Scope subclass: #SingleScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

SingleScope subclass: #LexicalScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

SingleScope subclass: #RunTimeScope
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Mode subclass: #Semantics
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

!Semantics commentStamp: 'NickPapoulias 7/23/2015 19:16' prior: 0!
semantics of hierarchy and reification!

Semantics subclass: #Hierarchy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Hierarchy subclass: #Flat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Hierarchy subclass: #Inheritance
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Semantics subclass: #Reification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Reification subclass: #Entity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Reification subclass: #Instance
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Modes'!

Object subclass: #RTVarAnalysis
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Case-Study'!

Object subclass: #Reflecto
	instanceVariableNames: ''
	classVariableNames: 'BaseMetaDict'
	poolDictionaries: ''
	category: 'Reflectogram-Graph-Points'!

!Reflecto class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/15/2015 15:55'!
baseMetaDict

	^ BaseMetaDict ifNil: [ BaseMetaDict := Dictionary new ]
	
	
	! !

!Reflecto class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 18:00'!
on: anEntity as: semanticsPair for: anEvent limitedTo: installationCondition when: userCondition do: userAction

	"single methods can be treated through class -- same for contexts"
	
	| reification hierarchy target event|
	
	anEvent isBehavior "allow the class of an Event to represent it but use an actual instance"
		ifTrue: [ event := anEvent new ]
		ifFalse: [ event := anEvent ].
	
	reification := semanticsPair key. "Instance -> Flat etc"
	hierarchy  := semanticsPair value.
	
	target := (reification = Instance) "This can also be done with Target on:given: replicating here for clarity" 
		ifTrue: [ ObjectTarget new target: anEntity  ] "treat as object"
		ifFalse: [ (Target resolve: anEntity) new target: anEntity ]. "treat as entity -- which also includes objects"
	
	target 
		setScopeFor: event; "major scope and sub scope"
		calculateAffectedMethodsGiven: hierarchy. 
	
	installationCondition
		ifNotNil: [ target limitAffectedMethodsGiven: installationCondition ].
	
	target
		userCondition: userCondition;
		userAction: userAction.
	
	^ (ReflectoGram of: anEntity) 
			addTarget: (target install; yourself);
			yourself! !

!Reflecto class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:39'!
on: anEntity for: anEvent limitedTo: installationCondition when: userCondition do: userAction
	
	| defaultSemanticPair |
	defaultSemanticPair := (Target resolve: anEntity) defaultSemanticPair.
	^ self 
		on: anEntity 
		as: defaultSemanticPair 
		for: anEvent 
		limitedTo: installationCondition 
		when: userCondition 
		do: userAction! !

!Reflecto class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:38'!
on: anEntity for: anEvent when: userCondition do: userAction
	
	^ self on: anEntity for: anEvent limitedTo: nil when: userCondition do: userAction
	
	
	! !

Reflecto subclass: #ReflectoGram
	instanceVariableNames: 'targetObject targets traced points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Graph-Points'!

!ReflectoGram class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 11:03'!
cleanUp

	super cleanUp.
	BaseMetaDict := nil.
	! !

!ReflectoGram class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/14/2015 19:49'!
of: anEntity

	^ self baseMetaDict at: anEntity ifAbsentPut: (self new on: anEntity)! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/15/2015 17:15'!
addTarget: aTarget

	targets add: (aTarget associatedReflectogram: self).! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/17/2015 19:49'!
cleanUp

	"recompile all targets"! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 17:30'!
disable

	targets do: [ :target | target disable ]! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 17:30'!
enable

	targets do: [ :target | target enable ]! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 10:52'!
initialize

	super initialize.
	targets := OrderedCollection new.
	points := nil.! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/14/2015 18:44'!
install! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:25'!
newPointFor: aTarget withDefaultAction: aBlockClosure

	^ 	ReflectoPoint new
			on: aTarget
			withDefaultAction: aBlockClosure.
	"traced ifTrue: [ self points add: point trace ]."
	 ! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 10:53'!
newPointFor: aTarget withDefaultAction: aBlockClosure in: aContext

	| point |
	point := 	ReflectoPoint new
			on: aTarget
			withDefaultAction: aBlockClosure
			in: aContext.
	traced ifTrue: [ self points add: point trace ].
	^ point ! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/15/2015 15:57'!
on: anEntity

	targetObject := anEntity.
	targets := OrderedCollection new. 
	traced := false. 
	points := OrderedCollection new.! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:54'!
on: anEntity as: semanticsPair for: anEvent limitedTo: installationCondition when: userCondition do: userAction

	^ self class on: anEntity as: semanticsPair for: anEvent limitedTo: installationCondition when: userCondition do: userAction! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:54'!
on: anEntity for: anEvent limitedTo: installationCondition when: userCondition do: userAction

	^ self class on: anEntity for: anEvent limitedTo: installationCondition when: userCondition do: userAction! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:54'!
on: anEntity for: anEvent when: userCondition do: userAction

	^ self class on: anEntity for: anEvent when: userCondition do: userAction! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 10:53'!
points

	^ points ifNil: [ points := OrderedCollection new ]! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 12:21'!
remove

	targets do: [ :target | target uninstall ].
	self class baseMetaDict removeKey: targetObject ifAbsent: [  ]! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/15/2015 16:03'!
traced: aBoolean

	traced := aBoolean.! !

!ReflectoGram methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 11:07'!
uninstall

	targets do: [ :target | target uninstall ].
	self class baseMetaDict removeKey: targetObject ! !

Reflecto subclass: #ReflectoPoint
	instanceVariableNames: 'target defaultAction traced tracedTime tracedReifications inContext returnValue reifications override'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Graph-Points'!

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:42'!
at: aName! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:42'!
at: aName put: aValue! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/15/2015 17:26'!
defaultAction

	^ defaultAction ! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:41'!
disable! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:41'!
enable! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:33'!
isEnabled! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:41'!
objectMetaLevel! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:02'!
on: anEntity as: semanticsPair for: anEvent limitedTo: installationCondition when: userCondition do: userAction

	^ self class on: anEntity as: semanticsPair for: anEvent limitedTo: installationCondition when: userCondition do: userAction! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:02'!
on: anEntity for: anEvent limitedTo: installationCondition when: userCondition do: userAction

	^ self class on: anEntity for: anEvent limitedTo: installationCondition when: userCondition do: userAction! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:54'!
on: anEntity for: anEvent when: userCondition do: userAction

	^ self class on: anEntity for: anEvent when: userCondition do: userAction! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:24'!
on: aTarget withDefaultAction: aBlockClosure

	override := false. "by default"
	target := aTarget.
	defaultAction := aBlockClosure.
	inContext := nil. "find it on run-time" ! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 12:30'!
on: aTarget withDefaultAction: aBlockClosure in: aContext

	override := false. "by default"
	target := aTarget.
	defaultAction := aBlockClosure.
	inContext := aContext ! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:33'!
override

	^ override	! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:33'!
override: aBoolean

	override := aBoolean	! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:42'!
perform: selector withArgument: arguments! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:41'!
processMetaLevel! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:06'!
reifications! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 10:41'!
remove! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/15/2015 17:25'!
returnValue

	^ returnValue! !

!ReflectoPoint methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:37'!
returnValue: aValue
	override := true.
	returnValue := aValue! !

Object subclass: #ReflectoGramVisualization
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Graph-Points'!

Object subclass: #Reifications
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Graph-Points'!

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:15'!
arguments



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:17'!
event



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:15'!
interpreterContext



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:15'!
interpreterContext: aContext

	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:17'!
method



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:17'!
name



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:15'!
receiver



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:15'!
selector



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:16'!
sender



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:16'!
trigger



	! !

!Reifications methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 11:16'!
value



	! !

Object subclass: #Target
	instanceVariableNames: 'scope hierarchy affectedMethods affectedClasses installCondition targetedObject associatedEvent associatedReflectogram userCondition userAction enabled falseCondition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

Target class
	instanceVariableNames: 'resolveDict'!

Target class
	instanceVariableNames: 'resolveDict'!

Target subclass: #RunTimeTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

RunTimeTarget subclass: #ContextTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Context-Process-Add-Ons'!

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
calculateAffectedMethodsGiven: anHierarchy

	affectedMethods := {targetedObject method} asOrderedCollection! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
defaultAffectedClasses

	^ {targetedObject method class} asOrderedCollection
	! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/3/2015 12:03'!
defaultAffectedHierarchy

	self shouldNotImplement 
	! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 09:32'!
isRunTime

	^ true! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:21'!
realMultiScopedEvent: anEvent

	self shouldNotImplement ! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:18'!
runTimeCondition
	
	^ (thisContext sender sender sender) = targetedObject ! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:21'!
simulatedMultiScopedEvent: anEvent

	self shouldNotImplement ! !

!ContextTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := RunTimeScope ! !

RunTimeTarget subclass: #ObjectTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

!ObjectTarget class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:34'!
defaultSemanticPair

	^ Instance -> Flat! !

!ObjectTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
defaultAffectedClasses
	
	^ {targetedObject class} asOrderedCollection ! !

!ObjectTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/16/2015 19:34'!
entityConditionGroupingInstances

	^ [ :object | object = targetedObject  ]! !

!ObjectTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 09:33'!
isRunTime
	^ true! !

!ObjectTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:19'!
runTimeCondition
	"todo: with a more static reification"
	^ thisContext sender sender sender receiver = targetedObject
	! !

!ObjectTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 17:16'!
simulatedMultiScopedEvent: anEvent

	anEvent scope = SimulatedMultiScope
		ifTrue: [ self shouldNotImplement ]
		ifFalse: [ scope := anEvent scope ]
	! !

!ObjectTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := RunTimeScope ! !

RunTimeTarget subclass: #ProcessTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Context-Process-Add-Ons'!

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
calculateAffectedMethodsGiven: anHierarchy

	"start with the suspended context and then just intercess on return and on send"

	targetedObject suspend.
	affectedMethods := {targetedObject suspendedContext method} asOrderedCollection! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
defaultAffectedClasses

	targetedObject suspend.
	^ {targetedObject suspendedContext method methodClass} asOrderedCollection ! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/3/2015 12:29'!
defaultAffectedHierarchy

	self shouldNotImplement ! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 09:32'!
isRunTime

	^ true! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:22'!
realMultiScopedEvent: anEvent

	self shouldNotImplement ! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 14:18'!
runTimeCondition
	
	^ Processor activeProcess = targetedObject ! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:22'!
simulatedMultiScopedEvent: anEvent

	self shouldNotImplement ! !

!ProcessTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := RunTimeScope ! !

Target subclass: #StructuralTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

StructuralTarget subclass: #ClassTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

!ClassTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
defaultAffectedClasses

	^ {targetedObject} asOrderedCollection ! !

!ClassTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/15/2015 18:29'!
entityConditionGroupingInstances
	
	^ [ :object | object class = targetedObject ]! !

!ClassTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := LexicalScope ! !

StructuralTarget subclass: #ImageTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

!ImageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/3/2015 12:21'!
defaultAffectedClasses

	^ ProtoObject allSubclasses! !

!ImageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/3/2015 12:09'!
defaultAffectedHierarchy
	
	^ ProtoObject allSubclasses! !

!ImageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/15/2015 17:10'!
entityConditionGroupingInstances
	
	^ [ :object | true ]! !

!ImageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := LexicalScope ! !

StructuralTarget subclass: #MethodTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

!MethodTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
calculateAffectedMethodsGiven: anHierarchy

	affectedMethods := {targetedObject} asOrderedCollection! !

!MethodTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/11/2015 13:46'!
defaultAffectedClasses
	
	^ targetedObject methodClass! !

!MethodTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 8/3/2015 12:10'!
defaultAffectedHierarchy
	
	self shouldNotImplement 
	! !

!MethodTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:22'!
realMultiScopedEvent: anEvent

	self shouldNotImplement ! !

!MethodTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/29/2015 16:22'!
simulatedMultiScopedEvent: anEvent

	self shouldNotImplement ! !

!MethodTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := LexicalScope ! !

StructuralTarget subclass: #PackageTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Targets'!

!PackageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/19/2015 14:15'!
defaultAffectedClasses
	
	^ targetedObject classes asOrderedCollection "be mindfull of what a set (default returned) can do here given the way = works for methods ..."! !

!PackageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/13/2015 12:49'!
entityConditionGroupingInstances
	
	^ [ :object | object class package = targetedObject ]! !

!PackageTarget methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 10/16/2015 12:35'!
singleScopedEvent: anEvent

	scope := LexicalScope ! !

!Target class methodsFor: 'accessing' stamp: 'NickPapoulias 10/11/2015 13:34'!
defaultSemanticPair

	^ Entity -> Flat! !

!Target class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/25/2015 13:09'!
on: anEntity

	^ self on: anEntity given: Entity! !

!Target class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 7/25/2015 13:08'!
on: anEntity given: reification
	
	(reification = Instance) 
		ifTrue: [ ^ ObjectTarget new target: anEntity  ] "treat as object"
		ifFalse: [ ^ (Target resolve: anEntity) new target: anEntity ]. "treat as entity -- which also includes objects"	! !

!Target class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/14/2015 20:37'!
resolve: anEntity
	
	self resolveDict keysAndValuesDo: [ :classKey : targetClass |
		(anEntity isKindOf: classKey) ifTrue: [ ^ targetClass ]		
	].

	^ ObjectTarget
	! !

!Target class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/14/2015 20:35'!
resolveDict
	
	^ resolveDict ifNil: [resolveDict := { Context -> ContextTarget . SmalltalkImage -> ImageTarget . CompiledMethod ->MethodTarget .RPackage -> PackageTarget . Process -> ProcessTarget . Class -> ClassTarget } asDictionary]  
! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/15/2015 17:16'!
associatedReflectogram: aReflectogram

	associatedReflectogram := aReflectogram! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/19/2015 13:48'!
calculateAffectedClassesGiven: anHierarchy

	scope = RealMultiScoped ifFalse: [  
	
		hierarchy := anHierarchy.
	
		anHierarchy = Flat
			ifTrue: [ affectedClasses := self defaultAffectedClasses ]
			ifFalse: [ affectedClasses := self defaultAffectedHierarchy ]
	
	] ifTrue: [  	
			affectedClasses := ProtoObject allSubclasses.
	]! !

!Target methodsFor: 'scoping-public' stamp: 'NickPapoulias 11/19/2015 17:13'!
calculateAffectedMethodsGiven: anHierarchy

	self calculateAffectedClassesGiven: anHierarchy.
	(associatedEvent isKindOf: ObjectCreation)
		ifTrue: [ affectedMethods := OrderedCollection new. 
				affectedClasses do: [ :class | (class class methodDict keys includes: #new)
															ifTrue: [ affectedMethods add: (class class >> #new) ] ] ] "simulated"
		ifFalse: [ affectedMethods := affectedClasses flatCollect: [ :class | class methodDict values ] ]
	! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 8/3/2015 12:18'!
defaultAffectedClasses

	self subclassResponsibility ! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 8/3/2015 12:20'!
defaultAffectedHierarchy

	| nextClass hierarchyCollection |
	self defaultAffectedClasses do: [ :aClass | 
		nextClass := aClass.
		hierarchyCollection := OrderedCollection new.
		[ nextClass ] whileNotNil: [ hierarchyCollection add: nextClass. nextClass := nextClass superclass ].
	].
	^ hierarchyCollection! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/20/2015 17:13'!
disable

	enabled := false.
	! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/20/2015 17:13'!
enable

	enabled := true.
	! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/13/2015 12:54'!
entityConditionGroupingInstances

	self shouldNotImplement ! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/20/2015 17:25'!
initialize

	super initialize.
	enabled := false. "following previous design"
	falseCondition := [:p | false ].! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/19/2015 13:36'!
install

	affectedMethods "currently this only depends " 
		do: [: method |  associatedEvent annotateMethod: method forTarget: self "the event should know" ].! !

!Target methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 11/20/2015 09:32'!
isRunTime

	^ false
! !

!Target methodsFor: 'scoping-public' stamp: 'NickPapoulias 11/19/2015 14:16'!
limitAffectedMethodsGiven: installationCondition

	installCondition := installationCondition.
	affectedMethods := affectedMethods select: installCondition.
	affectedClasses := (affectedMethods collect: [ :method | method methodClass ]) asSet.
	! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/20/2015 15:06'!
llmObject
	
	"^ llmObject ifNil: [ llmObject := LowLevelImplicitMetaObject new ]"! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 7/29/2015 16:02'!
realMultiScopedEvent: anEvent

	scope := anEvent scope! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/15/2015 17:17'!
reflectogram
		
		^ associatedReflectogram ! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/19/2015 13:47'!
setScopeFor: anEvent

	associatedEvent := anEvent.
	anEvent resolveScopeFor: self.! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 7/29/2015 16:02'!
simulatedMultiScopedEvent: anEvent

	scope := anEvent scope! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 7/29/2015 15:37'!
singleScopedEvent: anEvent

	self subclassResponsibility ! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 10/11/2015 13:46'!
target
	^ targetedObject! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 10/11/2015 13:46'!
target: anObject
	targetedObject := anObject! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/17/2015 14:47'!
uninstall
		
	associatedEvent uninstall.! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 10/11/2015 14:44'!
userAction
	^ userAction! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 10/11/2015 14:44'!
userAction: anObject
	userAction := anObject! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 11/20/2015 17:24'!
userCondition
	^ enabled ifTrue: [userCondition] ifFalse: [ falseCondition ]! !

!Target methodsFor: 'accessing' stamp: 'NickPapoulias 10/11/2015 14:44'!
userCondition: anObject
	userCondition := anObject! !

Object subclass: #View
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Case-Study'!

Object subclass: #WebApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reflectogram-Case-Study'!
